﻿--[[
	Copyright (c) 2009, dr_AllCOM3
    All rights reserved.

    You're allowed to use this addon, free of monetary charge,
    but you are not allowed to modify, alter, or redistribute
    this addon without express, written permission of the author.
]]

local addon = LibStub( "AceAddon-3.0" ):NewAddon( "DocsDebugRunes", "AceEvent-3.0", "AceConsole-3.0" )
DocsDebugRunes = addon
local L = LibStub( "AceLocale-3.0" ):GetLocale( "DocsDebugRunes" )
local LSM = LibStub("LibSharedMedia-3.0")

local tinsert = tinsert

local db, temp
local debug = false
local frames = {}
local locked = true
local maxRunicPower = 100
local maxHealth = UnitHealthMax( "player" )
local playerName = UnitName( "player" )
local deathStrikeTimeframe = 5
local deathStrikeHealPercent = 0.2
local deathStrikeHealPercentBase = 0.07
local masteryRating = GetMastery()
local bloodShieldPerMasteryRating = 6.25
local incomingDamage = {}

local localization = {
    ff = GetSpellInfo( 59921 ),
    bp = GetSpellInfo( 59879 ),
    km = GetSpellInfo( 51128 ),
    sd = GetSpellInfo( 49530 ),
    dt = GetSpellInfo( 63560 ),
    si = GetSpellInfo( 49572 ),
    rc = GetSpellInfo( 51462 ),
    re = GetSpellInfo( 81229 ),
    rime = GetSpellInfo( 59052 ),
    bs = GetSpellInfo( 77513 ),
    ds = GetSpellInfo( 49998 ),
}

local function p( text ) -- Print
	ChatFrame3:AddMessage( tostring( text ) )
end

local function lightenColor( r, g, b )
    local modifier = 1.5
    
    return r+( 1-r )/modifier, g+( 1-g )/modifier, b+( 1-b )/modifier
end

local function siValue( val ) -- Short numbers
    if val>=10000000 then
        return format( "%.1fm", val / 1000000 )
    elseif val>=1000000 then
        return format( "%.2fm", val / 1000000 )
    elseif val>=100000 then
        return format( "%.0fk", val / 1000 )
    elseif val>=1000 then
        return format( "%.1fk", val / 1000 )
    else
        return val
    end
end

local function round( num, idp ) --number, decimal places
	local mult = 10^( idp or 0 )
	return math.floor( num * mult + 0.5 ) / mult
end

local function formatTime( time )
    if time>60 then
        time = floor( time/60 ).."m"
    elseif time>=3 then
        time = floor( time )
    else
        time = round( time, 1 )
    end
    
    return time
end

local function createImprovedStatusbar( self, existingBar ) -- Creates a texture that acts like a statusbar. This gets rid of the ugly tiling.
    local function UpdateBar( self )
        local fraction = .01
        local range = self.MaxVal-self.MinVal 
        local value = self.Value-self.MinVal
        local barsize = self.Dim or 100
        
        if range>0 and value>0 and range>=value then
            fraction = value/range
        else
            fraction = .0001
        end
        
        if self.Orientation=="VERTICAL" then 
            self.Bar:SetHeight( barsize*fraction )
            self.Bar:SetTexCoord( 0, 1, 1-fraction, 1 )
        else 
            self.Bar:SetWidth( barsize*fraction ) 
            self.Bar:SetTexCoord( 0, fraction, 0, 1 )
        end
    end
    
    local function UpdateSize( self ) 
        if self.Orientation=="VERTICAL" then
            self.Dim = self:GetHeight()
        else
            self.Dim = self:GetWidth()
        end
        
        UpdateBar( self ) 
    end
    
    local function SetValue( self, value )
        if value>=self.MinVal and value<=self.MaxVal then self.Value = value end
        
        UpdateBar( self ) 
    end
    
    local function GetValue( self )
        return self.Value
    end
    
    local function SetMinMaxValues( self, minval, maxval )
        if maxval>minval then
            self.MinVal = minval
            self.MaxVal = maxval
        else 
            self.MinVal = 0
            self.MaxVal = 100
        end
        
        if self.Value>self.MaxVal then self.Value = self.MaxVal
        elseif self.Value<self.MinVal then self.Value = self.MinVal end
        
        UpdateBar( self ) 
    end
    
    
    local function GetMinMaxValues( self )
        return self.MinVal, self.MaxVal
    end
    
    
    local function SetOrientation( self, orientation ) 
        if orientation=="VERTICAL" then
            self.Orientation = orientation
            self.Bar:ClearAllPoints()
            self.Bar:SetPoint( "BOTTOMLEFT" )
            self.Bar:SetPoint( "BOTTOMRIGHT" )
        else
            self.Orientation = "HORIZONTAL"
            self.Bar:ClearAllPoints()
            self.Bar:SetPoint( "TOPLEFT" )
            self.Bar:SetPoint( "BOTTOMLEFT" )
        end
        
        UpdateSize( self )
    end
    
    local function SetStatusBarColor( self, r, g, b, a )
        self.Bar:SetVertexColor( r, g, b, a )
    end
    
    local function SetStatusBarTexture( self, texture )
        self.Bar:SetTexture( texture )
    end
    
    local statusbar
    if not existingBar then
        statusbar = CreateFrame( "Frame", nil, self )
    else
        statusbar = existingBar
    end
    --statusbar:SetPoint( "TOP", self, "TOP", 0, 0 )
    statusbar:SetWidth( db.width )
    statusbar:SetHeight( db.height )
    
    if not statusbar.Bar then
        temp = statusbar:CreateTexture( nil, "ARTWORK" )
    else
        temp = statusbar.Bar
    end
    statusbar.Bar = temp
    
    statusbar.Value = 100
    statusbar.MinVal = 0
    statusbar.MaxVal = 100
    statusbar.Orientation = "HORIZONTAL"
    statusbar.UpdateSize = UpdateSize
	statusbar.SetValue = SetValue
    statusbar.GetValue = GetValue
	statusbar.SetMinMaxValues = SetMinMaxValues
    statusbar.GetMinMaxValues = GetMinMaxValues
	statusbar.SetOrientation = SetOrientation
	statusbar.SetStatusBarColor = SetStatusBarColor
	statusbar.SetStatusBarTexture = SetStatusBarTexture
    
    --statusbar:SetFrameLevel( level )
    statusbar:SetOrientation( "HORIZONTAL" )
    statusbar:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
    --DocsUI_Nameplates.setBorder( statusbar )
	
	statusbar:SetScript( "OnSizeChanged", UpdateSize )
	UpdateSize( statusbar )
    
    return statusbar
end

local runes = {
    {
        { rune=1, ready = true, runeType = 1, duration = 10, start = 0 },    -- RUNETYPE_BLOOD = 1
        { rune=2, ready = true, runeType = 1, duration = 10, start = 0 },    -- RUNETYPE_DEATH = 2
    },
    {
        { rune=1, ready = true, runeType = 2, duration = 10, start = 0 },    -- RUNETYPE_FROST = 3
        { rune=2, ready = true, runeType = 2, duration = 10, start = 0 },    -- RUNETYPE_CHROMATIC = 4
    },
    {
        { rune=1, ready = true, runeType = 3, duration = 10, start = 0 },
        { rune=2, ready = true, runeType = 3, duration = 10, start = 0 },
    },
}

local colors = {
    { 0.77, 0.12, 0.23, 1 }, -- RUNETYPE_BLOOD = 1
    { 0.3, 0.8, 0.1, 1 }, -- RUNETYPE_DEATH = 2
    { 0, 0.4, 0.7, 1 }, -- RUNETYPE_FROST = 3
    { 0.51, 0.23, 0.65, 1 }, -- RUNETYPE_CHROMATIC = 4
}

local colorsCooldown = {
    { 1, 0, 0, 0.33 }, -- RUNETYPE_BLOOD = 1
    { 0, 1, 0, 0.33 }, -- RUNETYPE_DEATH = 2
    { 0, 0, 1, 0.33 }, -- RUNETYPE_FROST = 3
    { 1, 0, 1, 0.33 }, -- RUNETYPE_CHROMATIC = 4
}

function addon:Command( param )
	if param=="lock" then
        locked = not locked
        
        addon:CreateFrames()
    elseif param=="debug" then
        debug = not debug
        
        addon:CreateFrames()
        
        ChatFrame1:AddMessage( "DocsDebugRunes debug: "..tostring( debug ) )
    else
        InterfaceOptionsFrame_OpenToCategory( "DocsDebugRunes" )
    end
end

function addon:locked()
    locked = not locked
    
    addon:CreateFrames()
end

local function calculateDeathStrikeHeal( timestamp, amount )
    local time = GetTime()
    local damage = 0
    
    if timestamp and amount then tinsert( incomingDamage, { timestamp, amount } ) end
    
    for i=#incomingDamage,1,-1 do
        if incomingDamage[i][1]+deathStrikeTimeframe<time then
            tremove( incomingDamage, i )
        else
            damage = damage+incomingDamage[i][2]
        end
    end
    
    damage = max( maxHealth*deathStrikeHealPercentBase, damage*deathStrikeHealPercent*( 1+( 3*0.15 ) ) )
    frames.ds.amount = damage
    frames.ds.debug.amount = damage
end

-- onUpdate
local function onUpdateDarkTransformation( self, elapsed )
    if db.enableDarkTransformation then
        if not locked then return end
        
        local time = GetTime()
        
        if not self.expirationTime or self.expirationTime-time<=0 then
            self.timer:SetText( nil )
            self:Hide()
            
            return
        end
        
        self:SetValue( self.expirationTime-time )
        if self.count then
            self.timer:SetText( self.count )
        else
            self.timer:SetText( formatTime( round( self.expirationTime-time, 1 ) ) )
        end
    end
end

local function onUpdateDiseases( self, elapsed )
    if db.enableDiseases then
        if not locked then return end
        
        local time = GetTime()
        
        if not self.expirationTime or self.expirationTime-time<=0 then
            self.timer:SetText( nil )
            self.debug:SetText( nil )
            self:Hide()
            
            return
        end
        
        self:SetValue( self.expirationTime-time )
        self.timer:SetText( round( self.expirationTime-time ) )
        
        if debug then
            self.debug:SetText( formatTime( round( self.expirationTime-time, 1 ) ) .." | ".. formatTime( round( self.duration, 1 ) ) )
        end
    end
end

local function onUpdateRunes( self, elapsed )
    if not locked then return end
    
    local duration = self.duration+elapsed
    self.lastDuration = duration
    
    if duration>=self.max then
        self.debug:SetText( nil )
        self:SetScript( "OnUpdate", nil )
    else
        if debug then
            self.debug:SetText( formatTime( round( self.duration, 1 ) ) .." | ".. formatTime( round( self.max, 1 ) ) )
        end
        self.duration = duration
        self:SetValue( duration )
    end
end

local function onUpdateBloodShield( self, elapsed )
    if db.enableBloodShield then
        if not locked then return end
        
        local time = GetTime()
        
        if not self.amount or not self.expirationTime or self.amount<=0 or self.expirationTime-time<=0 then
            self.amount = 0
            self.text:SetText( nil )
            self.debug:SetText( nil )
            self:Hide()
            
            return
        end
        
        self:SetValue( self.expirationTime-time )
        self.text:SetText( siValue( round( self.amount, 0 ) ) )
        self.debug:SetText( "BS exact: "..self.amount )
    end
end

local function onUpdateDeathStrike( self, elapsed )
    if db.enableDeathStrikeHeal then
        if not locked then return end
        
        local time = GetTime()
        
        calculateDeathStrikeHeal()
        
        self:SetValue( self.amount )
        self.text:SetText( siValue( self.amount ) )
    end
end

function addon:CreateFrames()
    if not frames.bg then frames.bg = CreateFrame( "Frame", "DocsDebugRunesFrame", UIParent ) end
    frames.bg:SetWidth( db.width*2+db.spacing*2+db.spacing*3 )
    frames.bg:SetHeight( db.height*3+db.spacing*3+db.spacing*3 )
    frames.bg:SetBackdrop( { 
        bgFile = "Interface\\Buttons\\WHITE8X8", 
        edgeFile = LSM:Fetch( "border", db.border ), tile = false, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    } )
    frames.bg:SetBackdropBorderColor( 0.5, 0.5, 0.5, 1 )
    frames.bg:SetBackdropColor( 0.1, 0.1, 0.1, 1 )
    if not UnitAffectingCombat( "player" ) then frames.bg:SetAlpha( db.oocAlpha ) end
    db.frames.bg = db.frames.bg or {}
    frames.bg:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) } -- http://www.wowwiki.com/API_Frame_StartMoving
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.bg:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.bg.xOfs = x
            db.frames.bg.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.bg:EnableMouse( true )
        frames.bg:SetMovable( true )
    else
        frames.bg:EnableMouse( false )
        frames.bg:SetMovable( false )
    end
    frames.bg:ClearAllPoints()
    frames.bg:SetPoint( "CENTER", UIParent, "CENTER", db.frames.bg.xOfs or 0, db.frames.bg.yOfs or 0 )
    
    for i=1,6 do
        if not frames[i] then frames[i] = createImprovedStatusbar( frames.bg, frames[i] ) end
        frames[i]:SetWidth( db.width )
        frames[i]:SetHeight( db.height )
        frames[i]:SetStatusBarColor( 1, 1, 1, 1 )
        frames[i]:SetMinMaxValues( 0, 1 )
        frames[i]:SetValue( 1 )
        frames[i]:UpdateSize( frames[i] )
        
        if not frames[i].bg then frames[i].bg = frames[i]:CreateTexture( nil, "BACKGROUND" ) end
        frames[i].bg:SetAllPoints( frames[i] )
        frames[i].bg:SetTexture( db.texture )
        
        if not frames[i].debug then frames[i].debug = frames.bg:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
        frames[i].debug:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
        frames[i].debug:SetJustifyH( "CENTER" )
        frames[i].debug:SetJustifyV( "CENTER" )
        frames[i].debug:SetTextColor( 0.66, 0.66, 0.66, 1 )
        if i==1 or i==3 or i==5 then
            frames[i].debug:SetPoint( "RIGHT", frames[i], "LEFT", -db.spacing*2, 0 )
        else
            frames[i].debug:SetPoint( "LEFT", frames[i], "RIGHT", db.spacing*2, 0 )
        end
        if debug then frames[i].debug:Show() else frames[i].debug:Hide() end
        
        if not frames[i].glow then frames[i].glow = CreateFrame( "Frame", nil, frames[i] ) end
        frames[i].glow:SetAllPoints( frames[i] )
        frames[i].glow:Hide()
        frames[i].glow:SetScript( "OnUpdate", function( self )
            self.texture:SetAlpha( self.alpha )
            self.alpha = self.alpha-self.alpha*0.03
            if self.alpha<=0.1 then
                self:Hide()
            end
        end )
        frames[i].glow:SetScript( "OnShow", function( self )
            self.texture:SetAlpha( 1 )
            self.alpha = 1
        end )
        
        if not frames[i].glow.texture then frames[i].glow.texture = frames[i].glow:CreateTexture( nil, "OVERLAY" ) end
        frames[i].glow.texture:SetAllPoints( frames[i].glow )
        frames[i].glow.texture:SetTexture( "Interface\\Buttons\\WHITE8X8" )
        
        frames[i].start = 0
        frames[i].max = 0
        frames[i].duration = 0
        frames[i].lastDuration = 0
        frames[i].switched = false
        frames[i].lastUpdate = 0
        frames[i].onUpdate = onUpdateRunes
    end
    
    if not frames.glow then frames.glow = CreateFrame( "Frame", nil, frames.bg ) end
    local inset = 2
    frames.glow:SetPoint( "TOPLEFT", frames.bg, "TOPLEFT", inset, -inset )
    frames.glow:SetPoint( "BOTTOMRIGHT", frames.bg, "BOTTOMRIGHT", -inset, inset )
    frames.glow:Hide()
    frames.glow.visible = false
    
    if not frames.glow.texture then frames.glow.texture = frames.glow:CreateTexture( nil, "OVERLAY" ) end
    frames.glow.texture:SetAllPoints( frames.glow )
    frames.glow.texture:SetTexture( "Interface\\Addons\\DocsDebugRunes\\rcGlow" )
    frames.glow.texture:SetVertexColor( 1,1,1,0.4 )
    
    -- Rune order
    do
    for i=1,6 do
        frames[i]:ClearAllPoints()
    end
    
    if db.runeOrder==1 then
        frames[3]:SetPoint( "TOPLEFT", frames.bg, "TOPLEFT", db.spacing*2, -db.spacing*2 )
        frames[3]:SetStatusBarColor( unpack(colors[2] ) )
        frames[4]:SetPoint( "LEFT", frames[3], "RIGHT", db.spacing, 0 )
        frames[4]:SetStatusBarColor( unpack(colors[2] ) )
        
        frames[5]:SetPoint( "TOP", frames[3], "BOTTOM", 0, -db.spacing )
        frames[5]:SetStatusBarColor( unpack(colors[3] ) )
        frames[6]:SetPoint( "LEFT", frames[5], "RIGHT", db.spacing, 0 )
        frames[6]:SetStatusBarColor( unpack(colors[3] ) )
        
        frames[1]:SetPoint( "TOP", frames[5], "BOTTOM", 0, -db.spacing )
        frames[1]:SetStatusBarColor( unpack(colors[1] ) )
        frames[2]:SetPoint( "LEFT", frames[1], "RIGHT", db.spacing, 0 )
        frames[2]:SetStatusBarColor( unpack(colors[1] ) )
    elseif db.runeOrder==2 then
        frames[1]:SetPoint( "TOPLEFT", frames.bg, "TOPLEFT", db.spacing*2, -db.spacing*2 )
        frames[1]:SetStatusBarColor( unpack(colors[1] ) )
        frames[2]:SetPoint( "LEFT", frames[1], "RIGHT", db.spacing, 0 )
        frames[2]:SetStatusBarColor( unpack(colors[1] ) )
        
        frames[5]:SetPoint( "TOP", frames[1], "BOTTOM", 0, -db.spacing )
        frames[5]:SetStatusBarColor( unpack(colors[3] ) )
        frames[6]:SetPoint( "LEFT", frames[5], "RIGHT", db.spacing, 0 )
        frames[6]:SetStatusBarColor( unpack(colors[3] ) )
        
        frames[3]:SetPoint( "TOP", frames[5], "BOTTOM", 0, -db.spacing )
        frames[3]:SetStatusBarColor( unpack(colors[2] ) )
        frames[4]:SetPoint( "LEFT", frames[3], "RIGHT", db.spacing, 0 )
        frames[4]:SetStatusBarColor( unpack(colors[2] ) )
    elseif db.runeOrder==3 then
        frames[5]:SetPoint( "TOPLEFT", frames.bg, "TOPLEFT", db.spacing*2, -db.spacing*2 )
        frames[5]:SetStatusBarColor( unpack(colors[3] ) )
        frames[6]:SetPoint( "LEFT", frames[5], "RIGHT", db.spacing, 0 )
        frames[6]:SetStatusBarColor( unpack(colors[3] ) )
        
        frames[1]:SetPoint( "TOP", frames[5], "BOTTOM", 0, -db.spacing )
        frames[1]:SetStatusBarColor( unpack(colors[1] ) )
        frames[2]:SetPoint( "LEFT", frames[1], "RIGHT", db.spacing, 0 )
        frames[2]:SetStatusBarColor( unpack(colors[1] ) )
        
        frames[3]:SetPoint( "TOP", frames[1], "BOTTOM", 0, -db.spacing )
        frames[3]:SetStatusBarColor( unpack(colors[2] ) )
        frames[4]:SetPoint( "LEFT", frames[3], "RIGHT", db.spacing, 0 )
        frames[4]:SetStatusBarColor( unpack(colors[2] ) )
    end
    end
    
    -- Runic power
    do
    if not frames.rp then frames.rp = createImprovedStatusbar( frames.bg, frames.rp ) end
    frames.rp:SetWidth( db.width*2+db.spacing*1 )
    frames.rp:SetHeight( db.height )
    frames.rp:UpdateSize( frames.rp )
    frames.rp:SetStatusBarColor( 0, 0.82, 1 )
    frames.rp:SetMinMaxValues( 0, maxRunicPower )
    if db.enableRunicPower then frames.rp:Show() else frames.rp:Hide() end
    db.frames.rp = db.frames.rp or {}
    frames.rp:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) }
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.rp:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.rp.xOfs = x
            db.frames.rp.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.rp:EnableMouse( true )
        frames.rp:SetMovable( true )
    else
        frames.rp:EnableMouse( false )
        frames.rp:SetMovable( false )
    end
    frames.rp:ClearAllPoints()
    frames.rp:SetPoint( "TOP", frames.bg, "BOTTOM", db.frames.rp.xOfs or 0, db.frames.rp.yOfs or 0 )
    
    if not frames.rp.text then frames.rp.text = frames.rp:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.rp.text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.rp.text:SetJustifyH( "CENTER" )
    frames.rp.text:SetJustifyV( "CENTER" )
    frames.rp.text:SetTextColor( lightenColor( 0, 0.82, 1 ) )
    frames.rp.text:SetPoint( "CENTER", frames.rp, "CENTER", 0, 0 )
    
    if not frames.rp.bg then frames.rp.bg = frames.rp:CreateTexture( nil, "BACKGROUND" ) end
    frames.rp.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.rp.bg:SetAllPoints( frames.rp )
    frames.rp.bg:SetVertexColor( 0, 0.205, 0.25 )
    end
    
    -- Diseases
    do
    if not frames.bp then frames.bp = createImprovedStatusbar( frames.bg, frames.bp ) end
    frames.bp:SetWidth( db.width )
    frames.bp:SetHeight( db.height )
    frames.bp:UpdateSize( frames.bp )
    frames.bp:SetStatusBarColor( 0.5, 0, 0.6 )
    frames.bp.lastUpdate = 0
    frames.bp:SetScript( "OnUpdate", onUpdateDiseases )
    if db.enableDiseases and not locked then frames.bp:Show() else frames.bp:Hide() end
    db.frames.bp = db.frames.bp or {}
    frames.bp:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) }
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.bp:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.bp.xOfs = x
            db.frames.bp.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.bp:EnableMouse( true )
        frames.bp:SetMovable( true )
    else
        frames.bp:EnableMouse( false )
        frames.bp:SetMovable( false )
    end
    frames.bp:ClearAllPoints()
    frames.bp:SetPoint( "BOTTOMLEFT", frames.bg, "TOPLEFT", db.frames.bp.xOfs or db.spacing*2, db.frames.bp.yOfs or 0 )
    
    if not frames.bp.timer then frames.bp.timer = frames.bp:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.bp.timer:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.bp.timer:SetJustifyH( "CENTER" )
    frames.bp.timer:SetJustifyV( "CENTER" )
    frames.bp.timer:SetTextColor( lightenColor( 0.5, 0, 0.6 ) )
    frames.bp.timer:SetPoint( "CENTER", frames.bp, "TOP", 0, 0 )
    
    if not frames.bp.debug then frames.bp.debug = frames.bp:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.bp.debug:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
    frames.bp.debug:SetJustifyH( "CENTER" )
    frames.bp.debug:SetJustifyV( "CENTER" )
    frames.bp.debug:SetTextColor( 0.66, 0.66, 0.66 )
    frames.bp.debug:SetPoint( "RIGHT", frames.bp, "LEFT", -db.spacing*2, 0 )
    if debug then frames.bp.debug:Show() else frames.bp.debug:Hide() end
    
    if not frames.bp.bg then frames.bp.bg = frames.bp:CreateTexture( nil, "BACKGROUND" ) end
    frames.bp.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.bp.bg:SetAllPoints( frames.bp )
    frames.bp.bg:SetVertexColor( 0.125, 0, 0.15 )
    
    if not frames.ff then frames.ff = createImprovedStatusbar( frames.bg, frames.ff ) end
    frames.ff:SetWidth( db.width )
    frames.ff:SetHeight( db.height )
    frames.ff:UpdateSize( frames.ff )
    frames.ff:SetStatusBarColor( 0, 0.45, 0.6 )
    frames.ff.lastUpdate = 0
    frames.ff:SetScript( "OnUpdate", onUpdateDiseases )
    if db.enableDiseases and not locked then frames.ff:Show() else frames.ff:Hide() end
    db.frames.ff = db.frames.ff or {}
    frames.ff:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) }
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.ff:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.ff.xOfs = x
            db.frames.ff.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.ff:EnableMouse( true )
        frames.ff:SetMovable( true )
    else
        frames.ff:EnableMouse( false )
        frames.ff:SetMovable( false )
    end
    frames.ff:ClearAllPoints()
    frames.ff:SetPoint( "BOTTOMRIGHT", frames.bg, "TOPRIGHT", db.frames.ff.xOfs or -db.spacing*2, db.frames.ff.yOfs or 0 )
    
    if not frames.ff.timer then frames.ff.timer = frames.ff:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.ff.timer:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.ff.timer:SetJustifyH( "CENTER" )
    frames.ff.timer:SetJustifyV( "CENTER" )
    frames.ff.timer:SetTextColor( lightenColor( 0, 0.45, 0.6 ) )
    frames.ff.timer:SetPoint( "CENTER", frames.ff, "TOP", 0, 0 )
    
    if not frames.ff.debug then frames.ff.debug = frames.ff:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.ff.debug:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
    frames.ff.debug:SetJustifyH( "CENTER" )
    frames.ff.debug:SetJustifyV( "CENTER" )
    frames.ff.debug:SetTextColor( 0.66, 0.66, 0.66 )
    frames.ff.debug:SetPoint( "LEFT", frames.ff, "RIGHT", db.spacing*2, 0 )
    if debug then frames.ff.debug:Show() else frames.ff.debug:Hide() end
    
    if not frames.ff.bg then frames.ff.bg = frames.ff:CreateTexture( nil, "BACKGROUND" ) end
    frames.ff.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.ff.bg:SetAllPoints( frames.ff )
    frames.ff.bg:SetVertexColor( 0, 0.1125, 0.15 )
    end
    
    -- Dark Transformation
    do
    if not frames.dt then frames.dt = createImprovedStatusbar( frames.bg, frames.dt ) end
    frames.dt:SetWidth( db.width )
    frames.dt:SetHeight( db.height )
    frames.dt:UpdateSize( frames.dt )
    frames.dt:SetStatusBarColor( 0, 0.5, 0 )
    frames.dt.lastUpdate = 0
    frames.dt:SetScript( "OnUpdate", onUpdateDarkTransformation )
    if db.enableDarkTransformation and not locked then frames.dt:Show() else frames.dt:Hide() end
    db.frames.dt = db.frames.dt or {}
    frames.dt:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) }
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.dt:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.dt.xOfs = x
            db.frames.dt.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.dt:EnableMouse( true )
        frames.dt:SetMovable( true )
    else
        frames.dt:EnableMouse( false )
        frames.dt:SetMovable( false )
    end
    frames.dt:ClearAllPoints()
    frames.dt:SetPoint( "BOTTOM", frames.bg, "TOP", db.frames.dt.xOfs or 0, db.frames.dt.yOfs or db.spacing*5 )
    
    if not frames.dt.timer then frames.dt.timer = frames.dt:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.dt.timer:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.dt.timer:SetJustifyH( "CENTER" )
    frames.dt.timer:SetJustifyV( "CENTER" )
    frames.dt.timer:SetTextColor( lightenColor( 0.5, 0, 0.6 ) )
    frames.dt.timer:SetPoint( "CENTER", frames.dt, "TOP", 0, 0 )
    
    if not frames.dt.bg then frames.dt.bg = frames.dt:CreateTexture( nil, "BACKGROUND" ) end
    frames.dt.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.dt.bg:SetAllPoints( frames.dt )
    frames.dt.bg:SetVertexColor( 0.125, 0, 0.15 )
    end
    
    -- Procs
    do
    if not frames.proc then frames.proc = CreateFrame( "Frame", nil, frames.bg ) end
    frames.proc:SetWidth( db.width )
    frames.proc:SetHeight( db.width )
    if db.enableProcs and not locked then frames.proc:Show() else frames.proc:Hide() end
    db.frames.proc = db.frames.proc or {}
    frames.proc:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) }
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.proc:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.proc.xOfs = x
            db.frames.proc.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.proc:EnableMouse( true )
        frames.proc:SetMovable( true )
    else
        frames.proc:EnableMouse( false )
        frames.proc:SetMovable( false )
    end
    frames.proc:ClearAllPoints()
    frames.proc:SetPoint( "BOTTOM", frames.bg, "TOP", db.frames.proc.xOfs or 0, db.frames.proc.yOfs or db.spacing*5+db.height+db.spacing*2 )
    
    if not frames.proc.texture then frames.proc.texture = frames.proc:CreateTexture( nil, "ARTWORK" ) end
    frames.proc.texture:SetAllPoints( frames.proc )
    frames.proc.texture:SetVertexColor( 1,1,1 )
    frames.proc.texture:SetTexture( "Interface\\Icons\\Spell_chargenegative" )
    end
    
    -- Blood Shield
    do
    if not frames.bs then frames.bs = createImprovedStatusbar( frames.bg, frames.bs ) end
    frames.bs:SetWidth( db.width*2+db.spacing*1 )
    frames.bs:SetHeight( db.height )
    frames.bs:UpdateSize( frames.bs )
    frames.bs:SetStatusBarColor( 255/255, 120/255, 0/255 )
    frames.bs.lastUpdate = 0
    frames.bs:SetScript( "OnUpdate", onUpdateBloodShield )
    frames.bs.amount = 0
    if db.enableBloodShield and not locked then frames.bs:Show() else frames.bs:Hide() end
    db.frames.bs = db.frames.bs or {}
    frames.bs:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) }
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.bs:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.bs.xOfs = x
            db.frames.bs.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.bs:EnableMouse( true )
        frames.bs:SetMovable( true )
    else
        frames.bs:EnableMouse( false )
        frames.bs:SetMovable( false )
    end
    frames.bs:ClearAllPoints()
    frames.bs:SetPoint( "TOP", frames.bg, "BOTTOM", db.frames.bs.xOfs or 0, db.frames.bs.yOfs or -db.spacing*2 )
    
    if not frames.bs.text then frames.bs.text = frames.bs:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.bs.text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.bs.text:SetJustifyH( "CENTER" )
    frames.bs.text:SetJustifyV( "CENTER" )
    frames.bs.text:SetTextColor( lightenColor( 255/255, 120/255, 0/255 ) )
    frames.bs.text:SetPoint( "CENTER", frames.bs, "CENTER", 0, 0 )
    
    if not frames.bs.bg then frames.bs.bg = frames.bs:CreateTexture( nil, "BACKGROUND" ) end
    frames.bs.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.bs.bg:SetAllPoints( frames.bs )
    frames.bs.bg:SetVertexColor( 255/255/4, 120/255/4, 0/255/4 )
    
    if not frames.bs.debug then frames.bs.debug = frames.bs:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.bs.debug:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
    frames.bs.debug:SetJustifyH( "CENTER" )
    frames.bs.debug:SetJustifyV( "CENTER" )
    frames.bs.debug:SetTextColor( 0.66, 0.66, 0.66 )
    frames.bs.debug:SetPoint( "LEFT", frames.bs, "RIGHT", db.spacing*2, 0 )
    frames.bs.debug.amount = 0
    if debug then frames.bs.debug:Show() else frames.bs.debug:Hide() end
    end
    
    -- Death Strike Heal
    do
    if not frames.ds then frames.ds = createImprovedStatusbar( frames.bg, frames.ds ) end
    frames.ds:SetWidth( db.width*2+db.spacing*1 )
    frames.ds:SetHeight( db.height )
    frames.ds:UpdateSize( frames.ds )
    frames.ds:SetStatusBarColor( 120/255, 255/255, 0/255 )
    frames.ds.amount = 1
    frames.ds.lastUpdate = 0
    frames.ds:SetScript( "OnUpdate", onUpdateDeathStrike )
    if db.enableDeathStrikeHeal and not locked then frames.ds:Show() else frames.ds:Hide() end
    db.frames.ds = db.frames.ds or {}
    frames.ds:SetScript( "OnMouseDown", function( self )
            if locked then return end
            
            self.tempPointsFrame = { self:GetPoint( 1 ) }
            
            self:StartMoving()
            
            self.tempPointsScreen = { self:GetPoint( 1 ) }
        end)
    frames.ds:SetScript( "OnMouseUp", function( self, button )
            if locked then return end
            
            local tempPoints = { self:GetPoint( 1 ) }
            
            self:StopMovingOrSizing()
            
            local x = tempPoints[4]-self.tempPointsScreen[4]
            local y = tempPoints[5]-self.tempPointsScreen[5]
            
            x = x+self.tempPointsFrame[4]
            y = y+self.tempPointsFrame[5]
            
            db.frames.ds.xOfs = x
            db.frames.ds.yOfs = y
            
            addon:CreateFrames()
        end )
    frames.bg:SetClampedToScreen( true )
    if not locked then
        frames.ds:EnableMouse( true )
        frames.ds:SetMovable( true )
    else
        frames.ds:EnableMouse( false )
        frames.ds:SetMovable( false )
    end
    frames.ds:ClearAllPoints()
    frames.ds:SetPoint( "TOP", frames.bg, "BOTTOM", db.frames.ds.xOfs or 0, db.frames.ds.yOfs or -db.spacing*2-db.height-db.spacing*2 )
    
    
    if not frames.ds.text then frames.ds.text = frames.ds:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.ds.text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.ds.text:SetJustifyH( "CENTER" )
    frames.ds.text:SetJustifyV( "CENTER" )
    frames.ds.text:SetTextColor( lightenColor( 120/255, 255/255, 0/255 ) )
    frames.ds.text:SetPoint( "CENTER", frames.ds, "CENTER", 0, 0 )
    
    if not frames.ds.bg then frames.ds.bg = frames.ds:CreateTexture( nil, "BACKGROUND" ) end
    frames.ds.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.ds.bg:SetAllPoints( frames.ds )
    frames.ds.bg:SetVertexColor( 120/255/4, 255/255/4, 0/255/4, 1 )
    
    if not frames.ds.debug then frames.ds.debug = frames.ds:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.ds.debug:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
    frames.ds.debug:SetJustifyH( "CENTER" )
    frames.ds.debug:SetJustifyV( "CENTER" )
    frames.ds.debug:SetTextColor( 0.66, 0.66, 0.66 )
    frames.ds.debug:SetPoint( "RIGHT", frames.ds, "LEFT", -db.spacing*2, 0 )
    frames.ds.debug.amount = 0
    if debug then frames.ds.debug:Show() else frames.ds.debug:Hide() end
    end
    
    -- Tutorial
    do
    if not frames.tutorial then frames.tutorial = CreateFrame( "Frame", nil, frames.bg ) end
    if locked then frames.tutorial:Hide() else frames.tutorial:Show() end
    
    if not frames.tutorial.move then frames.tutorial.move = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.move:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+8, "THINOUTLINE" )
    frames.tutorial.move:SetJustifyH( "CENTER" )
    frames.tutorial.move:SetJustifyV( "CENTER" )
    frames.tutorial.move:SetTextColor( 1, 1, 1 )
    frames.tutorial.move:SetPoint( "RIGHT", frames.bg, "TOPLEFT", -40, 80 )
    frames.tutorial.move:SetText( L["Config mode\nYou can move the frames now!"].."\n/ddr" )
    
    if not frames.tutorial.close then frames.tutorial.close = CreateFrame( "Button", nil, frames.tutorial, "OptionsButtonTemplate" ) end
    frames.tutorial.close:SetWidth( 80 )
    frames.tutorial.close:SetHeight( 22 )
    frames.tutorial.close:SetPoint( "BOTTOM", frames.tutorial.move, "TOP", 0, 5 )
    frames.tutorial.close:SetText( CLOSE )
    frames.tutorial.close:SetScript( "OnClick", function( self )
        locked = not locked
        
        addon:CreateFrames()
    end )
    
    if not frames.tutorial.options then frames.tutorial.options = CreateFrame( "Button", nil, frames.tutorial, "OptionsButtonTemplate" ) end
    frames.tutorial.options:SetWidth( 120 )
    frames.tutorial.options:SetHeight( 22 )
    frames.tutorial.options:SetPoint( "TOP", frames.tutorial.move, "BOTTOM", 0, -5 )
    frames.tutorial.options:SetText( L["Options"] )
    frames.tutorial.options:SetScript( "OnClick", function( self )
        InterfaceOptionsFrame_OpenToCategory( "DocsDebugRunes" )
    end )
    
    if not frames.tutorial.bg then frames.tutorial.bg = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.bg:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.bg:SetJustifyH( "CENTER" )
    frames.tutorial.bg:SetJustifyV( "CENTER" )
    frames.tutorial.bg:SetTextColor( 1, 1, 1 )
    frames.tutorial.bg:SetPoint( "LEFT", frames.bg, "RIGHT", 0, 0 )
    frames.tutorial.bg:SetText( L["Runes"] )
    
    if not frames.tutorial.rp then frames.tutorial.rp = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.rp:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.rp:SetJustifyH( "CENTER" )
    frames.tutorial.rp:SetJustifyV( "CENTER" )
    frames.tutorial.rp:SetTextColor( 1, 1, 1 )
    frames.tutorial.rp:SetPoint( "LEFT", frames.rp, "RIGHT", 0, 0 )
    frames.tutorial.rp:SetText( L["Runic Power"] )
    
    if not frames.tutorial.bp then frames.tutorial.bp = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.bp:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.bp:SetJustifyH( "CENTER" )
    frames.tutorial.bp:SetJustifyV( "CENTER" )
    frames.tutorial.bp:SetTextColor( 1, 1, 1 )
    frames.tutorial.bp:SetPoint( "RIGHT", frames.bp, "LEFT", 0, 0 )
    frames.tutorial.bp:SetText( localization.bp )
    
    if not frames.tutorial.ff then frames.tutorial.ff = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.ff:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.ff:SetJustifyH( "CENTER" )
    frames.tutorial.ff:SetJustifyV( "CENTER" )
    frames.tutorial.ff:SetTextColor( 1, 1, 1 )
    frames.tutorial.ff:SetPoint( "LEFT", frames.ff, "RIGHT", 0, 0 )
    frames.tutorial.ff:SetText( localization.ff )
    
    if not frames.tutorial.dt then frames.tutorial.dt = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.dt:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.dt:SetJustifyH( "CENTER" )
    frames.tutorial.dt:SetJustifyV( "CENTER" )
    frames.tutorial.dt:SetTextColor( 1, 1, 1 )
    frames.tutorial.dt:SetPoint( "LEFT", frames.dt, "RIGHT", 0, 0 )
    frames.tutorial.dt:SetText( L["Dark Transformation"] )
    
    if not frames.tutorial.proc then frames.tutorial.proc = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.proc:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.proc:SetJustifyH( "CENTER" )
    frames.tutorial.proc:SetJustifyV( "CENTER" )
    frames.tutorial.proc:SetTextColor( 1, 1, 1 )
    frames.tutorial.proc:SetPoint( "RIGHT", frames.proc, "LEFT", 0, 0 )
    frames.tutorial.proc:SetText( L["Procs"] )
    
    if not frames.tutorial.bs then frames.tutorial.bs = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.bs:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.bs:SetJustifyH( "CENTER" )
    frames.tutorial.bs:SetJustifyV( "CENTER" )
    frames.tutorial.bs:SetTextColor( 1, 1, 1 )
    frames.tutorial.bs:SetPoint( "RIGHT", frames.bs, "LEFT", 0, 0 )
    frames.tutorial.bs:SetText( L["Blood Shield"] )
    
    if not frames.tutorial.ds then frames.tutorial.ds = frames.tutorial:CreateFontString( nil, "OVERLAY", "GameFontNormal" ) end
    frames.tutorial.ds:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorial.ds:SetJustifyH( "CENTER" )
    frames.tutorial.ds:SetJustifyV( "CENTER" )
    frames.tutorial.ds:SetTextColor( 1, 1, 1 )
    frames.tutorial.ds:SetPoint( "LEFT", frames.ds, "RIGHT", 0, 0 )
    frames.tutorial.ds:SetText( L["Death Strike"] )
    end
end

-- Update diseases
local debuffs = { target = {}, player = {}, pet = {} }
local function aura( unit )
    local counter = 1
    
    if unit=="target" then
        wipe( debuffs[unit] )
        
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( unit, i, "HARMFUL" )
            if name and unitCaster=="player" and name==localization.ff then
                debuffs[unit].ff = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.bp then
                debuffs[unit].bp = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif not name then
                break
            end
        end
        
        if db.enableDiseases and debuffs[unit].ff then --"Frost Fever"
            local time = GetTime()
            
            frames.ff:SetMinMaxValues( 0, debuffs[unit].ff[6] )
            frames.ff:SetValue( time-( debuffs[unit].ff[7]-time ) )
            
            frames.ff.expirationTime = debuffs[unit].ff[7]
            frames.ff.duration = debuffs[unit].ff[6]
            frames.ff:SetMinMaxValues( 0, debuffs[unit].ff[6] )
            frames.ff:Show()
        elseif locked then
            frames.ff:Hide()
        end
        
        if db.enableDiseases and debuffs[unit].bp then --"Blood Plague"
            local time = GetTime()
            
            frames.bp:SetMinMaxValues( 0, debuffs[unit].bp[6] )
            frames.bp:SetValue( time-( debuffs[unit].bp[7]-time ) )
            
            frames.bp.expirationTime = debuffs[unit].bp[7]
            frames.bp.duration = debuffs[unit].bp[6]
            frames.bp:SetMinMaxValues( 0, debuffs[unit].bp[6] )
            frames.bp:Show()
        elseif locked then
            frames.bp:Hide()
        end
    elseif unit=="player" then
        wipe( debuffs[unit] )
        
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( unit, i, "HELPFUL" )
            
            if name and unitCaster=="player" and name==localization.km then
                debuffs[unit].km = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.sd then
                debuffs[unit].sd = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.rc then
                debuffs[unit].rc = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.rime then
                debuffs[unit].rime = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.bs then
                debuffs[unit].bs = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif not name then
                break
            end
        end
        
        if db.enableProcs and debuffs[unit].rime then --"Freezing Fog"
            frames.proc.texture:SetTexture( debuffs[unit].rime[3] )
            frames.proc:Show()
        elseif db.enableProcs and debuffs[unit].km then --"Killing Machine"
            frames.proc.texture:SetTexture( debuffs[unit].km[3] )
            frames.proc:Show()
        elseif db.enableProcs and debuffs[unit].sd then --"Sudden Doom"
            frames.proc.texture:SetTexture( debuffs[unit].sd[3] )
            frames.proc:Show()
        elseif locked then
            frames.proc:Hide()
        end
        
        if debuffs[unit].rc then
            if not frames.glow.visible then
                frames.glow:Show()
                frames.glow.visible = true
            end
        else
            if frames.glow.visible then
                frames.glow:Hide()
                frames.glow.visible = false
            end
        end
        
        if db.enableBloodShield and debuffs[unit].bs then
            local time = GetTime()
            
            frames.bs:SetMinMaxValues( 0, debuffs[unit].bs[6] )
            frames.bs:SetValue( time-( debuffs[unit].bs[7]-time ) )
            
            frames.bs.expirationTime = debuffs[unit].bs[7]
            frames.bs.duration = debuffs[unit].bs[6]
            frames.bs:SetMinMaxValues( 0, debuffs[unit].bs[6] )
            
            frames.bs:Show()
        elseif locked then
            frames.bs.amount = 0
            
            frames.bs:Hide()
        end
        
        maxHealth = UnitHealthMax( "player" )
        frames.ds:SetMinMaxValues( maxHealth*deathStrikeHealPercentBase, maxHealth*deathStrikeHealPercent )
    elseif unit=="pet" then
        wipe( debuffs[unit] )
        
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( unit, i, "HELPFUL" )
            
            if name and name==localization.dt then
                debuffs[unit].dt = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and name==localization.si then
                debuffs[unit].si = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif not name then
                break
            end
        end
        
        if db.enableDarkTransformation and debuffs[unit].dt then
            local time = GetTime()
            
            frames.dt:SetMinMaxValues( 0, debuffs[unit].dt[6] )
            frames.dt:SetValue( time-( debuffs[unit].dt[7]-time ) )
            
            frames.dt.count = nil
            frames.dt.expirationTime = debuffs[unit].dt[7]
            frames.dt.duration = debuffs[unit].dt[6]
            frames.dt:SetMinMaxValues( 0, debuffs[unit].dt[6] )
            
            frames.dt:SetStatusBarColor( 0.5, 0.25, 0, 1 )
            frames.dt.timer:SetTextColor( lightenColor( 0.5, 0.25, 0, 1 ) )
            frames.dt:Show()
        elseif db.enableDarkTransformation and debuffs[unit].si then
            local time = GetTime()
            
            frames.dt:SetMinMaxValues( 0, debuffs[unit].si[6] )
            frames.dt:SetValue( time-( debuffs[unit].si[7]-time ) )
            
            frames.dt.count = debuffs[unit].si[4]
            frames.dt.expirationTime = debuffs[unit].si[7]
            frames.dt.duration = debuffs[unit].si[6]
            frames.dt:SetMinMaxValues( 0, debuffs[unit].si[6] )
            
            frames.dt:SetStatusBarColor( 0, 0.5, 0, 1 )
            frames.dt.timer:SetTextColor( lightenColor( 0, 0.5, 0, 1 ) )
            frames.dt:Show()
        elseif locked then
            frames.dt:Hide()
        end
    end
end

-- On Rune Update
local function onRuneCooldownUpdate( self, rune )
    if not rune or rune>6 then
        return
    end
    
    local place, pool, modificator
    if rune==1 then
        pool = 1
        place = 1
    elseif rune==2 then
        pool = 1
        place = 2
    elseif rune==3 then
        pool = 2
        place = 1
    elseif rune==4 then
        pool = 2
        place = 2
    elseif rune==5 then
        pool = 3
        place = 1
    elseif rune==6 then
        pool = 3
        place = 2
    end
    if place==1 then modificator = 1 else modificator = -1 end
    
    local runeOther = rune+modificator
    
    local time = GetTime()
    
    local start, duration, runeReady = GetRuneCooldown( rune )
    runes[pool][place].ready = runeReady
    runes[pool][place].duration = time-start
    runes[pool][place].start = start
    runes[pool][place].max = duration
    
    local start, duration, runeReady = GetRuneCooldown( runeOther )
    runes[pool][place+modificator].ready = runeReady
    runes[pool][place+modificator].duration = time-start
    runes[pool][place+modificator].start = start
    runes[pool][place+modificator].max = duration
    
    if time-runes[pool][1].start<time-runes[pool][2].start then
        rune, runeOther = runeOther, rune
    end
    
    if runes[pool][place].ready then
        frames[rune]:SetMinMaxValues( 0, 1 )
        frames[rune]:SetValue( 1 )
        frames[rune]:SetScript( "OnUpdate", nil )
        frames[rune].debug:SetText( nil )
        frames[rune]:SetAlpha( 1 )
        if frames[rune].lastDuration>frames[rune].max+0.5 then
            frames[rune].glow:Show()
        end
        frames[rune].lastDuration = 0
    else
        frames[rune].duration = time-runes[pool][place].start
        frames[rune].max = runes[pool][place].max
        frames[rune].start = runes[pool][place].start
        frames[rune]:SetMinMaxValues( 0, frames[rune].max )
        frames[rune]:SetScript( "OnUpdate", frames[rune].onUpdate )
        frames[rune]:SetAlpha( 0.4 )
        
        if frames[rune].duration<0 then frames[rune]:SetValue( 0 ) end
    end
    frames[rune]:SetStatusBarColor( unpack( colors[runes[pool][place].runeType] ) )
    frames[rune].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place].runeType] ) )
    
    if runes[pool][place+modificator].ready then
        frames[runeOther]:SetMinMaxValues( 0, 1 )
        frames[runeOther]:SetValue( 1 )
        frames[runeOther]:SetScript( "OnUpdate", nil )
        frames[runeOther].debug:SetText( nil )
        frames[runeOther]:SetAlpha( 1 )
        if frames[runeOther].lastDuration>frames[runeOther].max+0.5 then
            frames[runeOther].glow:Show()
        end
        frames[runeOther].lastDuration = 0
    else
        frames[runeOther].duration = time-runes[pool][place+modificator].start
        frames[runeOther].max = runes[pool][place+modificator].max
        frames[runeOther].start = runes[pool][place+modificator].start
        frames[runeOther]:SetMinMaxValues( 0, frames[runeOther].max )
        frames[runeOther]:SetScript( "OnUpdate", frames[runeOther].onUpdate )
        frames[runeOther]:SetAlpha( 0.4 )
        
        if frames[runeOther].duration<0 then frames[runeOther]:SetValue( 0 ) end
    end
    frames[runeOther]:SetStatusBarColor( unpack( colors[runes[pool][place+modificator].runeType] ) )
    frames[runeOther].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place+modificator].runeType] ) )
end

-- Events
function addon:UNIT_POWER( event, unit, type )
    if not unit then unit = "player" end
    
    if db.enableRunicPower and unit=="player" then
        local power = UnitMana( unit )
        
        frames.rp:SetValue( power )
        frames.rp.text:SetText( power )
    end
end

function addon:RUNE_POWER_UPDATE( event, rune )
    onRuneCooldownUpdate( self, rune )
end

function addon:RUNE_TYPE_UPDATE( event, rune )
    if not rune or rune>6 then return end
    
    local runeType = GetRuneType( rune )
    
    local place, pool, modificator
    if rune==1 then
        pool = 1
        place = 1
    elseif rune==2 then
        pool = 1
        place = 2
    elseif rune==3 then
        pool = 2
        place = 1
    elseif rune==4 then
        pool = 2
        place = 2
    elseif rune==5 then
        pool = 3
        place = 1
    elseif rune==6 then
        pool = 3
        place = 2
    end
    if place==1 then modificator = 1 else modificator = -1 end
    
    runes[pool][place].runeType = runeType
    
    local runeOther = rune+modificator
    local time = GetTime()
    if time-runes[pool][1].start<time-runes[pool][2].start then
        rune, runeOther = runeOther, rune
    end
    
    frames[rune]:SetStatusBarColor( unpack( colors[runes[pool][place].runeType] ) )
    frames[rune].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place].runeType] ) )
    frames[runeOther]:SetStatusBarColor( unpack( colors[runes[pool][place+modificator].runeType] ) )
    frames[runeOther].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place+modificator].runeType] ) )
end

function addon:UNIT_AURA( event, unit )
    if unit=="target" or unit=="player" or unit=="pet" then aura( unit ) end
end

function addon:PLAYER_TARGET_CHANGED( event )
    aura( "target" )
end

function addon:PLAYER_ENTERING_WORLD( event )
    for i=1,6 do
        addon:RUNE_TYPE_UPDATE( nil, i )
    end
    for i=1,6 do
        addon:RUNE_POWER_UPDATE( nil, i )
    end
end

function addon:PLAYER_REGEN_DISABLED( event )
    frames.bg:SetAlpha( 1 )
    
    if db.enableDeathStrikeHeal then frames.ds:Show() end
end

function addon:PLAYER_REGEN_ENABLED( event )
    frames.bg:SetAlpha( db.oocAlpha )
    
    if locked then frames.ds:Hide() end
end

function addon:PLAYER_TALENT_UPDATE( event )
    maxRunicPower = UnitManaMax( "player" )
    frames.rp:SetMinMaxValues( 0, maxRunicPower )
end

function addon:COMBAT_LOG_EVENT_UNFILTERED( event, ... )
    local timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2, spellId, spellName, spellSchool = select(1, ...)
    
    if destName==playerName then
        if event=="SWING_DAMAGE" then
            local amount, overkill, school, resisted, blocked, absorbed = select(12, ...)
            
            if school==1 and absorbed and absorbed>0 then
                frames.bs.amount = frames.bs.amount-absorbed
                if frames.bs.amount<0 then frames.bs.amount = 0 end
            end
            
            calculateDeathStrikeHeal( GetTime(), amount )
        elseif event=="SPELL_DAMAGE" then
            local amount, overkill, school, resisted, blocked, absorbed = select(15, ...)
            
            if school==1 and absorbed and absorbed>0 then
                frames.bs.amount = frames.bs.amount-absorbed
                if frames.bs.amount<0 then frames.bs.amount = 0 end
            end
            
            calculateDeathStrikeHeal( GetTime(), amount )
        elseif event=="RANGE_DAMAGE" then
            local amount, overkill, school, resisted, blocked, absorbed = select(15, ...)
            
            if school==1 and absorbed and absorbed>0 then
                frames.bs.amount = frames.bs.amount-absorbed
                if frames.bs.amount<0 then frames.bs.amount = 0 end
            end
            
            calculateDeathStrikeHeal( GetTime(), amount )
        elseif event=="SWING_MISSED" then
            local missType, amountMissed = select(12, ...)
            
            if missType=="ABSORB" and amountMissed~=nil then
                frames.bs.amount = frames.bs.amount-amountMissed
                if frames.bs.amount<0 then frames.bs.amount = 0 end
            end
        elseif event=="SPELL_MISSED" then
            local missType, amountMissed = select(15, ...)
            
            if spellSchool==1 and missType=="ABSORB" and amountMissed~=nil then
                frames.bs.amount = frames.bs.amount-amountMissed
                if frames.bs.amount<0 then frames.bs.amount = 0 end
            end
        elseif event=="RANGE_MISSED" then
            local missType, amountMissed = select(15, ...)
            
            if spellSchool==1 and missType=="ABSORB" and amountMissed~=nil then
                frames.bs.amount = frames.bs.amount-amountMissed
                if frames.bs.amount<0 then frames.bs.amount = 0 end
            end
        end
    end
    
    if sourceName==playerName then
        if event=="SPELL_CAST_SUCCESS" then
            --local spellId, spellName, spellSchool = select(15, ...)
            
            if spellName==localization.ds then
                frames.bs.amount = frames.bs.amount+frames.ds.amount*masteryRating*bloodShieldPerMasteryRating/100
            end
        end
        
        if event=="SPELL_HEAL" then
            local amount, overhealing, absorbed, critical = select(15, ...)
            
            if debug and spellName==localization.ds then
                frames.ds.debug:SetText( "Last DS heal: "..amount.."\nEst. DS heal: "..round( frames.ds.debug.amount, 1 ) )
                frames.ds.debug.amount = 0
            else
                frames.ds.debug:SetText( nil )
            end
        end
        
        if event=="SPELL_AURA_APPLIED" then
            local auraType = select(15, ...)
            
            if spellName==localization.bs then
                
            end
        end
    end
    
end

addon.defaults = { profile = {
    tutorialMove = false,
    
    width = 30,
    height = 10,
    spacing = 4,
    fontsize = 12,
    oocAlpha = 1,
    texture = "DocsDebugRunes", -- Standard: "Interface\\TargetingFrame\\UI-StatusBar"
    font = "Segoe UI", -- Standard: "Fonts\\FRIZQT__.TTF"
    border = "DocsDebugRunes", -- Standard: "Interface\\DialogFrame\\UI-DialogBox-Border"
    runeOrder = 1,
    enableDiseases = true,
    enableDarkTransformation = true,
    enableRunicPower = true,
    enableProcs = true,
    enableBloodShield = true,
    enableDeathStrikeHeal = true,
} }

-- OnInitialize
function addon:OnInitialize()
    if select( 2, UnitClass( "player" ) )~="DEATHKNIGHT" then return end
    
    -- DB
	self.db = LibStub( "AceDB-3.0" ):New( "DocsDebugRunesDB", self.defaults )
    db = addon.db.profile
    
    -- Options
    self:SpawnOptions()
    LibStub( "AceConfig-3.0" ):RegisterOptionsTable( "DocsDebugRunes", addon.options )
    LibStub( "AceConfigDialog-3.0" ):AddToBlizOptions( "DocsDebugRunes", "DocsDebugRunes" )
    
    -- Profile options
    local AceDBOptions = LibStub( "AceDBOptions-3.0", true )
	if AceDBOptions then
		self.options.args.profiles = AceDBOptions:GetOptionsTable( self.db )
		self.options.args.profiles.order = 999
	end
    
    -- Command
    self:RegisterChatCommand( "ddr", "Command" )
    
    LSM:Register("font", "Segoe UI",            [[Interface\Addons\DocsDebugRunes\font.ttf]])
    LSM:Register("statusbar", "DocsDebugRunes", [[Interface\Addons\DocsDebugRunes\statusbar]])
    LSM:Register("border", "DocsDebugRunes",             [[Interface\Addons\DocsDebugRunes\caith]])
end
-- OnEnable
function addon:OnEnable()
    if select( 2, UnitClass( "player" ) )~="DEATHKNIGHT" then return end
    RuneFrame:Hide()
    RuneFrame:HookScript( "OnShow", function( self ) self:Hide() end )
    
    db.frames = db.frames or {}
    maxRunicPower = UnitManaMax( "player" )
    
    if not db.firstRun then
        locked = not locked
        
        db.firstRun = 1
    end
    
    addon:CreateFrames()
    
    addon:RegisterEvent( "RUNE_POWER_UPDATE" )
    addon:RegisterEvent( "RUNE_TYPE_UPDATE" )
    addon:RegisterEvent( "UNIT_AURA" )
    addon:RegisterEvent( "PLAYER_TARGET_CHANGED" )
    addon:RegisterEvent( "PLAYER_ENTERING_WORLD" )
    addon:RegisterEvent( "PLAYER_REGEN_DISABLED" )
    addon:RegisterEvent( "PLAYER_REGEN_ENABLED" )
    addon:RegisterEvent( "UNIT_POWER" )
    addon:RegisterEvent( "COMBAT_LOG_EVENT_UNFILTERED" )
    addon:UNIT_POWER()
end
function addon:OnDisable()
	
end
